/*
  Function : internal ADC initialization
  Arguments : 
    channels - (BIT7 BIT6 BIT5 BIT4 BIT3 BIT2 BIT1 BIT0)
      BITx 
        '1' - channel x (ADx) enabled
        '0' - channel x (ADx) disabled
     
    prescaler - PRESC[5:0]
      The ADC clock range is between MCK/2, if PRESCAL is 0, and MCK/128, 
      if PRESCAL is set to 63 (0x3F). PRESCAL must be programmed in order 
      to provide an ADC clock frequency according to the parameters given 
      in the Product definition section.
  
  Returns : none
*/
void ADC_Init(unsigned char channels, unsigned int prescaler);

/*
  Function : starts ADC
  Arguments : nonr
  Returns : none
*/
void ADC_StartConversion(void);
/*
  Function :  allows to determine if a new sample is ready to be read
  Arguments : 
    channel - 0 to 7
  Returns : 
    0: No sample available on the corresponding channel.  
    1: New sample on the corresponding channel is available.
*/
unsigned char ADC_DataReady(unsigned char channel);

/*
  Function : reads corresponding to channel data register
  Arguments : 
    channel - 0 to 7
  Returns : 
    10 bits of sampled value on the corresponding channel.
*/
int ADC_ReadChannel(unsigned char channel);

