// ----------------------------------------------------------------------------
//         ATMEL Microcontroller Software Support  -  ROUSSET  -
// ----------------------------------------------------------------------------
// DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
// DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
// OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ----------------------------------------------------------------------------
//  File Name           : interrupt_Usart.c
//  Object              : USART Interrupt Management
//
//  1.0 24/Jun/04 JPP   : Creation
//  1.1 29/Aug/05 JPP   : Update AIC definion
//  1.3 29/Jun/06 JPP   : Clean for genaration
// ----------------------------------------------------------------------------

// Include Standard LIB  files
#include "Project.h"
#include "C:\Program Files\IAR Systems\Embedded Workbench 4.0 Evaluation\ARM\inc\intrinsics.h"


#define USART_INTERRUPT_LEVEL		7
#define AT91_BAUD_RATE				115200
//#define GPS_BAUD_RATE				5720 // 6250
#define dGPS_BAUD_RATE				9600 // *12

int GPS_BAUD_RATE=dGPS_BAUD_RATE;

static const char atmel_header[]=
{
"\n\r  *** ATMEL Usart IRQ ***\n\r"
"Copyright (C) 2002 ATMEL Corporations Version: 1.0\n\r"
};

//* \fn    AT91F_US_Baudrate
//* \brief Calculate the baudrate
//* Standard Asynchronous Mode : 8 bits , 1 stop , no parity
#define AT91C_US_ASYNC_MODE ( AT91C_US_USMODE_NORMAL + \
                        AT91C_US_NBSTOP_1_BIT + \
                        AT91C_US_PAR_NONE + \
                        AT91C_US_CHRL_8_BITS + \
                        AT91C_US_CLKS_CLOCK )

#define  ComBufSize ( 1 << 9 )
char ComBuf0[ComBufSize];
int ComBufIn0;
int ComBufOut0;

void ComClr0()
{ ComBufIn0=1; ComBufOut0=0;
}

int ComCnt0()  /*  get how many bytes are in the buffer*/
{ return ( (ComBufIn0-ComBufOut0-1) & (ComBufSize-1) );
}

unsigned char GetPort0() /*  get a character from the buffer*/
{  return  ComBuf0[ (++ComBufOut0) & (ComBufSize-1) ];
}

void ComPut0(unsigned char Data)
{ ComBuf0[ (ComBufIn0++) & (ComBufSize-1) ] = Data;
}

char bpc;

 void bp(char lbpc )
 {
 bpc=lbpc;
 }

//*----------------------------------------------------------------------------
//* Function Name       : Usart_c_irq_handler
//* Object              : C handler interrupt function called by the interrupts
//*                       assembling routine
//* Input Parameters    : <RTC_pt> time rtc descriptor
//* Output Parameters   : increment count_timer0_interrupt
//*----------------------------------------------------------------------------
void Usart_c_irq_handler(void) // GPS
{
	AT91PS_USART USART_pt = AT91C_BASE_US0;
	unsigned int status;
	//* get Usart status register
	status = USART_pt->US_CSR;
	if ( status & AT91C_US_RXRDY){
		//* Get byte and send
//		AT91F_US_PutChar (USART_pt, AT91F_US_GetChar(USART_pt));
          char lbpc;
  __disable_interrupt();
            lbpc=AT91F_US_GetChar(USART_pt);
//            if( ( (0x80&lbpc) != 0 )|( lbpc<0xa) )
//              bp(lbpc);
		ComPut0(lbpc);
  __enable_interrupt();
	}

	if ( status & AT91C_US_OVRE) {
		//* clear US_RXRDY
		 AT91F_US_GetChar(USART_pt);
		 AT91F_US_PutChar (USART_pt, 'O');
	}

	//* Check error
	if ( status & AT91C_US_PARE) {
		 AT91F_US_PutChar (USART_pt, 'P');
	}

	if ( status & AT91C_US_FRAME) {
		 AT91F_US_PutChar (USART_pt, 'F');
	}

	if ( status & AT91C_US_TIMEOUT){
		USART_pt->US_CR = AT91C_US_STTTO;
		 AT91F_US_PutChar (USART_pt, 'T');
	}

	//* Reset the satus bit
	 USART_pt->US_CR = AT91C_US_RSTSTA;
}


char ComBuf1[ComBufSize];
int ComBufIn1;
int ComBufOut1;

void ComClr1()
{ ComBufIn1=1; ComBufOut1=0;
}

int ComCnt1()  /*  get how many bytes are in the buffer*/
{ return ( (ComBufIn1-ComBufOut1-1) & (ComBufSize-1) );
}

unsigned char GetPort1() /*  get a character from the buffer*/
{  return  ComBuf1[ (++ComBufOut1) & (ComBufSize-1) ];
}

void ComPut1(unsigned char Data)
{ ComBuf1[ (ComBufIn1++) & (ComBufSize-1) ] = Data;
}


#define  ComBufSize1Tx ( 1 << 9 )
char ComBuf1Tx[ComBufSize1Tx];
int ComBufIn1Tx;
int ComBufOut1Tx;

void ComClr1Tx()
{ ComBufIn1Tx=1; ComBufOut1Tx=0;
}

int ComCnt1Tx()  /*  get how many bytes are in the buffer*/
{ return ( (ComBufIn1Tx-ComBufOut1Tx-1) & (ComBufSize1Tx-1) );
}

unsigned char GetPort1Tx() /*  get a character from the buffer*/
{
		if (!ComCnt1Tx())
  {  __disable_interrupt();
//      forthwdt();
    __enable_interrupt();
    return 0;
 }
//		ser_bTransmission = 0;

  return  ComBuf1Tx[ (++ComBufOut1Tx) & (ComBufSize1Tx-1) ];
}

void ComPut1Tx(unsigned char Data)
{  while( (ComBufSize1Tx-1) == ComCnt1Tx()  ) QGSM_INT_ON() ;
 __disable_interrupt; //div
 ComBuf1Tx[ (ComBufIn1Tx++) & (ComBufSize1Tx-1) ] = Data;
 __enable_interrupt;  //div

}

void usart1_send(unsigned char Data)
{
#ifndef   debugpwd
  AT91PS_USART USART_pt = AT91C_BASE_US1;
USART_pt->US_IER = AT91C_US_TXRDY | AT91C_US_RXRDY;
ComPut1Tx(Data);
//ser_bTransmission = 1;
USART_pt->US_IER = AT91C_US_TXRDY | AT91C_US_RXRDY;
#endif
}

int vCTS=0;
QGSM_INT_ON()
{
    AT91PS_USART USART_pt1 = AT91C_BASE_US1;
    if(vCTS)
            {
		    if(!(AT91F_PIO_GetInput(AT91C_BASE_PIOA) & AT91C_PIO_PA25))
                {   	//AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_US1);
                while (!(AT91C_BASE_DBGU->DBGU_CSR & AT91C_US_TXRDY));
                AT91C_BASE_DBGU->DBGU_THR = '#';
		        if (ComCnt1Tx())
                        { USART_pt1->US_THR = GetPort1Tx(); }
		        else
		            {
					    while (!(AT91C_BASE_DBGU->DBGU_CSR & AT91C_US_TXRDY));
		                AT91C_BASE_DBGU->DBGU_THR = '$';
		            }
                vCTS=0;
                }
            }
}
int vvvvv;

//*----------------------------------------------------------------------------
//* Function Name       : Usart_c_irq_handler
//* Object              : C handler interrupt function called by the interrupts
//*                       assembling routine
//* Input Parameters    : <RTC_pt> time rtc descriptor
//* Output Parameters   : increment count_timer0_interrupt
//*----------------------------------------------------------------------------
void Usart_c_irq_handler1(void) // GSM
{
	AT91PS_USART USART_pt1 = AT91C_BASE_US1;
	unsigned int status;
	//* get Usart status register
	status = USART_pt1->US_CSR;
	if ( status & AT91C_US_RXRDY){
          char lbpc;
		//* Get byte and send
//		AT91F_US_PutChar (USART_pt1, AT91F_US_GetChar(USART_pt1));
                            lbpc=AT91F_US_GetChar(USART_pt1);
//  __disable_interrupt();
//            if( ( (0x80&lbpc) != 0 )|( lbpc<0xa) )
//              bp(lbpc);
		ComPut1(lbpc);
//  __enable_interrupt();

	}

	if (status & AT91C_US_TXRDY)
	{       if(vCTS)
                  { USART_pt1->US_IDR = AT91C_US_TXRDY;
                return;
                }
		if (!ComCnt1Tx())
		                {
		                USART_pt1->US_IDR = AT91C_US_TXRDY;
                        //		ser_bTransmission = 0;
		                }
		else
		{	if((AT91F_PIO_GetInput(AT91C_BASE_PIOA) & AT91C_PIO_PA25))
                        {    USART_pt1->US_IDR = AT91C_US_TXRDY;
                        //  AT91F_AIC_DisableIt (AT91C_BASE_AIC, AT91C_ID_US1);
//                         vCTS=1;
                        }
			else        USART_pt1->US_THR = GetPort1Tx();
		}
	}

	if ( status & AT91C_US_OVRE) {
		//* clear US_RXRDY
//		 AT91F_US_GetChar(USART_pt1);
//		 AT91F_US_PutChar (USART_pt1, 'O');
	}

	//* Check error
	if ( status & AT91C_US_PARE) {
//		 AT91F_US_PutChar (USART_pt1, 'P');
	}

	if ( status & AT91C_US_FRAME) {
//		 AT91F_US_PutChar (USART_pt1, 'F');
	}

	if ( status & AT91C_US_TIMEOUT){
		USART_pt1->US_CR = AT91C_US_STTTO;
//		 AT91F_US_PutChar (USART_pt1, 'T');
	}

	//* Reset the satus bit
	 USART_pt1->US_CR = AT91C_US_RSTSTA;
}

//*-------------------------- External Function -------------------------------

void GSM_BAUD(int baud)
{ 	AT91PS_USART COM1 = AT91C_BASE_US1;

  // First, enable the clock of the PIOB
    	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_US1 ) ;
	// Usart Configure
      AT91F_US_Configure (COM1, AT91B_MCK, AT91C_US_ASYNC_MODE, baud, 0);

	// Enable usart
	COM1->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;

    //* Enable USART IT error and RXRDY
    	AT91F_US_EnableIt(COM1,AT91C_US_TIMEOUT | AT91C_US_FRAME | AT91C_US_OVRE |AT91C_US_RXRDY);

    	//* open Usart 1 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_US1, USART_INTERRUPT_LEVEL,AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Usart_c_irq_handler1);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_US1);

        ComClr1();
        ComClr1Tx();


// 	AT91F_US_SendFrame(COM1,(char *)atmel_header,sizeof(atmel_header),0,0);
}

void USART_DIS1()
{
	AT91C_BASE_US1->US_CR = 0;
    	AT91F_US_DisableIt(AT91C_BASE_US1,AT91C_US_TIMEOUT | AT91C_US_FRAME | AT91C_US_OVRE |AT91C_US_RXRDY);
	AT91F_AIC_DisableIt (AT91C_BASE_AIC, AT91C_ID_US1);
  
  AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, AT91C_PA22_TXD1 ) ;
  AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, AT91C_PA22_TXD1 ); 
  
}


//*----------------------------------------------------------------------------
//* Function Name       : Usart_init
//* Object              : USART initialization
//* Input Parameters    : none
//* Output Parameters   : TRUE
//*----------------------------------------------------------------------------
void Usart_init1 ( void ) // GSM
//* Begin
{

                AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		((unsigned int) AT91C_PA21_RXD1    ) |
 		((unsigned int) AT91C_PA22_TXD1    )  /*  |
 		((unsigned int) AT91C_PA24_RTS1    ) |
 		((unsigned int) AT91C_PA25_CTS1   )  */ , // Peripheral A
 		0); // Peripheral B


    GSM_BAUD(AT91_BAUD_RATE);


//* End
}

void Usart_init( void ) // GPS
//* Begin
{
	AT91PS_USART COM0 = AT91C_BASE_US0;

 	//* Configure PIO controllers to periph mode
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		((unsigned int) AT91C_PA5_RXD0    ) |
 		((unsigned int) AT91C_PA6_TXD0    ) |
 		((unsigned int) AT91C_PA7_RTS0    ) |
 		((unsigned int) AT91C_PA8_CTS0    ), // Peripheral A
 		0); // Peripheral B


   	// First, enable the clock of the PIOB
    	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_US0 ) ;
	// Usart Configure
      AT91F_US_Configure (COM0, AT91B_MCK, AT91C_US_ASYNC_MODE, GPS_BAUD_RATE, 0);

	// Enable usart
	COM0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;

    //* Enable USART IT error and RXRDY
    	AT91F_US_EnableIt(COM0,AT91C_US_TIMEOUT | AT91C_US_FRAME | AT91C_US_OVRE |AT91C_US_RXRDY);

    	//* open Usart 1 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_US0, USART_INTERRUPT_LEVEL,AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Usart_c_irq_handler);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_US0);

//	AT91F_US_PutChar (COM0,'X');
        
//	for(;;) AT91F_US_PutChar (COM0,'X');

// 	AT91F_US_SendFrame(COM0,(char *)atmel_header,sizeof(atmel_header),0,0);

        ComClr0();

//* End
}


